"""
Raccoon: Virus Genome Alignment QC Curation Package

A comprehensive toolkit for curating virus genome and sub-genomic sequence alignments
and flagging various QC issues including clustered SNPs, convergent SNPs, ADAR edits,
and APOBEC3 edits.
"""

_program = "raccoon"
__version__ = "1.0.0"
